using System;
using System.Data;

namespace BusinessValidation
{
	/// <summary>
	/// Base class for all data validators.
	/// </summary>
	[Serializable ()]
	public abstract class Validator
	{
		protected string _tableName;
		protected string _columnName;

		protected string _errMessage;

		protected Validator (string t, string c, string m)
		{
			_tableName = t;
			_columnName = c;
			_errMessage = m;
		}

		// Responsible for wiring events.
		public void ConnectTo (DataSet ds)
		{
			ds.Tables[_tableName].ColumnChanged +=new 
				DataColumnChangeEventHandler(Validator_ColumnChanged);
		}

		abstract public void Validate (DataSet ds, 
			DataRow r, DataColumn c, object newNal);

		private void Validator_ColumnChanged(object sender, 
			DataColumnChangeEventArgs e)
		{
			if (e.Column.ColumnName == _columnName)
				Validate (e.Row.Table.DataSet,
					e.Row, e.Column, e.ProposedValue);
		}
	}

	[Serializable ()]
	public class RangeValidator : Validator
	{
		private object _minValue;
		private object _maxValue;

		public RangeValidator (string t, string c, string m,
			object min,
			object max) :
			base (t,c, m)
		{
			_minValue = min;
			_maxValue = max;
		}

		public override void Validate (DataSet ds, 
			DataRow r, DataColumn c, object newVal)
		{
			IComparable curr = newVal as IComparable;
			if ((curr.CompareTo (_minValue) < 0) // Too Small,
				|| (curr.CompareTo (_maxValue) > 0)) // too large
			{
				r.SetColumnError (c, _errMessage);
				r.RowError = _errMessage;
			} 
			else
			{
				r.SetColumnError (c, "");
				r.RowError = "";
			}
		}
	}

	[Serializable ()]
	public class ComputeValidator : Validator
	{
		private string _formula; // The formula.
		private object _result; // What to expect
		private string _filterColumn; // which column to use for the filter.
		private bool _markAllRows; // Mark all rows matching the filter?

		public ComputeValidator (string t, string c, string m,
			string formula,
			string filterColumnName,
			object result,
			bool markAll) :
			base (t, c, m)
		{
			_formula = formula;
			_filterColumn = filterColumnName;
			_result = result;
			_markAllRows = markAll;
		}

		public override void Validate(DataSet ds, DataRow r, DataColumn c, object newNal)
		{
			System.Text.StringBuilder filter= new System.Text.StringBuilder(_filterColumn);
			if (_filterColumn.Length > 0)
			{
				filter.Append ("='");
				filter.Append (r[_filterColumn].ToString ());
				filter.Append ("'");
			}
			object result = r.Table.Compute (_formula, filter.ToString ());
			
			if (_markAllRows)
			{
				DataView v = new DataView (r.Table);
				v.RowFilter = filter.ToString ();

				if (_result.Equals (result))
				{
					foreach (DataRowView row in v)
					{
						row.Row.SetColumnError (c, "");
						row.Row.RowError = "";
					}
				} 
				else			
				{
					foreach (DataRowView row in v)
					{
						row.Row.SetColumnError (c, _errMessage);
						row.Row.RowError = _errMessage;
					}
				}
			} 
			else
			{
				if (_result.Equals (result))
				{
					r.SetColumnError (c, "");
					r.RowError = "";
				} 
				else			
				{
					r.SetColumnError (c, _errMessage);
					r.RowError = _errMessage;
				}
			}
			
		}

	}
}
